#define F_CPU 8000000UL

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <stdint.h>
#include <util/delay.h>
#include "ntc.h"

#define SEG_A  PB0
#define SEG_B  PB1
#define SEG_C  PB2
#define SEG_D  PB3
#define SEG_E  PB4
#define SEG_F  PB5
#define SEG_G  PB6
#define SEG_DP PA7
#define CA_1   PA2
#define CA_2   PA1
#define CA_3   PA0
#define PUMP   PA6
#define HEATER PA5

const uint8_t digits[] PROGMEM = {0x3f, 0x06, 0x5b, 0x4f, 0x66, 0x6d, 0x7d, 0x07, 0x7f, 0x6f};

volatile uint8_t displaybuffer[] = {0xff, 0xff, 0xff};
volatile uint8_t displaycounter;

volatile uint16_t adcbuffer = 0;
volatile uint8_t adccounter = 0;

uint8_t temp = 0;

int main(void)
{
    DDRB = 0x7f;
    PORTB = 0x7f;
    DDRA = (1 << CA_3) | (1 << CA_2) | (1 << CA_1) | (1 << HEATER) | (1 << PUMP) | (1 << SEG_DP);

    TIMSK = (1 << TOIE0);
    TCCR0B = (1 << CS01) | (1 << CS00);

    DIDR0 = (1 << ADC3D) | (1 << AREFD);
    ADCSRB = (1 << REFS2);
    ADMUX = (1 << REFS0) | (1 << REFS1) | (1 << MUX0) | (1 << MUX1);
    ADCSRA = (1 << ADEN) | (1 << ADSC) | (1 << ADATE) | (1 << ADIE) | (1 << ADPS2) | (1 << ADPS2);

    sei();

    while(1)
    {
        if(adccounter == 64)
        {
            uint8_t adc8 = adcbuffer >> 8;
            // displaybuffer[0] = pgm_read_byte(&digits[display % 10]);
            // displaybuffer[1] = pgm_read_byte(&digits[(display / 10) % 10]);
            // displaybuffer[2] = pgm_read_byte(&digits[(display / 100) % 10]);
            
            if(adc8 < 60 || adc8 > 250)
            {
                PORTA &= ~(1 << HEATER);
                displaybuffer[0] = 0x50;
                displaybuffer[1] = 0x50;
                displaybuffer[2] = 0x79;
            }
            else
            {
                uint8_t pretab = pgm_read_byte(&ntc[temp]);
                uint8_t mindist = 255;
                if(pretab < adc8)
                {
                   mindist = adc8 - pretab;
                }
                else
                {
                   mindist = pretab - adc8;
                }
                mindist -= 1;
                for(uint8_t i = 0; i < 57; i++)
                {
                    uint8_t tab = pgm_read_byte(&ntc[i]);
                    if(tab <= adc8)
                    {
                        if(adc8 - tab < mindist)
                        {
                            mindist = adc8 - tab;
                            temp = i;
                        }
                    }
                    else if(tab > adc8)
                    {
                        if(tab - adc8 < mindist)
                        {
                            mindist = tab - adc8;
                            temp = i;
                        }
                    }
                }

                displaybuffer[0] = 0x63;
                displaybuffer[2] = pgm_read_byte(&digits[(temp / 10) % 10]);
                displaybuffer[1] = pgm_read_byte(&digits[temp % 10]);

                if(temp > 50 && (PORTA & (1 << HEATER)))
                {
                    PORTA &= ~(1 << HEATER);
                }
                else if(temp < 45 && !(PORTA & (1 << HEATER)))
                {
                    PORTA |= (1 << HEATER);
                }
            }

            adcbuffer = 0;
            adccounter = 0;

            _delay_ms(200);
        }
    }

    return 0;
}

ISR(TIMER0_OVF_vect)
{
    displaycounter++;
    displaycounter %= 3;

    PORTA &= ~((1 << CA_1) | (1 << CA_2) | (1 << CA_3));
    PORTA |= (1 << displaycounter);

    PORTB = ~(displaybuffer[displaycounter] & 0x7f);
    if(displaybuffer[displaycounter] & 0x80)
    {
        PORTA &= ~(1 << SEG_DP);
    }
    else
    {
        PORTA |= (1 << SEG_DP);
    }

    PINA = (1 << PUMP);
}

ISR(ADC_vect)
{
    if(adccounter < 64)
    {
        adcbuffer += ADC;
        adccounter++;
    }
}
